#!/usr/bin/php -Cq
<?php

$argc = $_SERVER['argc'];
$argv = $_SERVER['argv'];

$root_path = dirname($argv[0]);
require_once($root_path.'/config.php');

$fp = fopen('php://stdin', 'rt');
if ($fp == 0) {
    writelog('Failed to open php://stdin!');
    exit;
}

echo "+ OK\n";

while (!feof($fp)) {
    $buf = trim(fgets($fp, 4096));
    if (feof($fp)) break;
    writelog(">> $buf");
    $aCmd = explode(' ', $buf);
    if ($aCmd === false) continue;
    $cmd = $aCmd[0];
    if (strcasecmp($cmd, 'QUIT') == 0) {
        $rbuf = '+ QUIT!';
        echo "$rbuf\n";
        writelog("<< $rbuf");
        break;
    }
    if (strcasecmp($cmd, 'LOGIN') == 0) {
        $user = base64_decode($aCmd[1]);
        $pass = base64_decode($aCmd[2]);
        if ($using_smtp_auth)
            $auth_result = CheckAuthSMTP($user, $pass);
        else
            $auth_result = CheckAuthPOP($user, $pass);
        if (!$auth_result) {
            $rbuf = '- ERR';
            echo "$rbuf\n";
            writelog("<< $rbuf");
            continue;
        }
        unset($aSession);
        writelog("User $user login!");
        $aSession['tm'] = time();
        $aSession['user'] = $user;
        $sid = saveSessionData($aSession);
        if ($sid === false)
            $rbuf = '- Failed to save session data!';
        else
            $rbuf = "+ $sid";
        echo "$rbuf\n";
        writelog("<< $rbuf");
        continue;
    }
    $sid = $aCmd[1];
    unset($aSession);
    $aSession = getSessionData($sid);
    if ($aSession === false) {
        $rbuf = "! No session for $sid, please LOGIN first!";
        echo "$rbuf\n";
        writelog("<< $rbuf");
        continue;
    }
    $user = $aSession['user'];
    writelog("User: $user");
    $now = time();
    if ($now >= $aSession['tm'])
        $len = $now - $aSession['tm'];
    else
        $len = $now;
    if ($len > $session_timeout) {
        deleteSession($sid);
        $rbuf = "! Session timeout for $user/$sid, please LOGIN again!";
        echo "$rbuf\n";
        writelog("<< $rbuf");
        continue;
    }
    if (strcasecmp($cmd, 'LOGOUT') == 0) {
        deleteSession($sid);
        $rbuf = "+ LOGOUT for $user/$sid!";
        echo "$rbuf\n";
        writelog("<< $rbuf");
        continue;
    }
    if (strcasecmp($cmd, 'STATUS') == 0) {
        $shcmd = $root_path.'/scripts/check.sh';
        $pipe = popen($shcmd, 'r');
        if (!$pipe) {
            $rbuf = "- popen($shcmd) failed!";
            echo "$rbuf\n";
            writelog("<< $rbuf");
            continue;
        }
        $pid = trim(fread($pipe, 1024));
        pclose($pipe);
        $aServerData = getServerData();
        $bandwidth = base64_encode($aServerData['bandwidth']);
        $size = base64_encode($aServerData['size']);
        $ch = getChannelData();
        $channel = base64_encode($ch);
        $fname = base64_encode($server_file);
        if ($pid !== '')
            $rbuf = "+ $pid $bandwidth $size $channel $fname";
        else
            $rbuf = "+ - $bandwidth $size $channel $fname";
        echo "$rbuf\n";
        writelog("<< $rbuf");
        continue;
    }
    if (strcasecmp($cmd, 'START') == 0) {
        $shcmd = $root_path.'/scripts/check.sh';
        $pipe = popen($shcmd, 'r');
        if (!$pipe) {
            $rbuf = "- popen($shcmd) failed!";
            echo "$rbuf\n";
            writelog("<< $rbuf");
            continue;
        }
        $pid = trim(fread($pipe, 1024));
        pclose($pipe);
        if ($pid !== '') {
            $rbuf = "- Process exist: $pid!";
            echo "$rbuf\n";
            writelog("<< $rbuf");
            continue;
        }
        if (!isset($aCmd[3])) {
            $rbuf = "- Syntax: START sid bandwidth screensize";
            echo "$rbuf\n";
            writelog("<< $rbuf");
            continue;
        }
        $bandwidth = base64_decode($aCmd[2]);
        $size = base64_decode($aCmd[3]);
        $channel = getChannelData();
        $shcmd = $root_path.'/scripts/start.sh "'.$bandwidth.'" "'.$size.'" "'.$channel.'" "'.$server_file.'"';
        $pipe = popen($shcmd, 'r');
        if (!$pipe) {
            $rbuf = "- popen($shcmd) failed!";
            echo "$rbuf\n";
            writelog("<< $rbuf");
            continue;
        }
        pclose($pipe);
        $shcmd = $root_path.'/scripts/check.sh';
        $pipe = popen($shcmd, 'r');
        if (!$pipe) {
            $rbuf = "- popen($shcmd) failed!";
            echo "$rbuf\n";
            writelog("<< $rbuf");
            continue;
        }
        $pid = trim(fread($pipe, 1024));
        pclose($pipe);
        if ($pid !== '')
            $rbuf = "+ Process started: $pid!";
        else
            $rbuf = "- Failed to start the process!";
        echo "$rbuf\n";
        saveServerData(array('bandwidth' => $bandwidth, 'size' => $size));
        writelog("<< $rbuf");
        continue;
    }
    if (strcasecmp($cmd, 'STOP') == 0) {
        $shcmd = $root_path.'/scripts/check.sh';
        $pipe = popen($shcmd, 'r');
        if (!$pipe) {
            $rbuf = "- popen($shcmd) failed!";
            echo "$rbuf\n";
            writelog("<< $rbuf");
            continue;
        }
        $pid = trim(fread($pipe, 1024));
        pclose($pipe);
        if ($pid === '') {
            $rbuf = "- Process not exist!";
            echo "$rbuf\n";
            writelog("<< $rbuf");
            continue;
        }
        if ($pid == 0) {
            $rbuf = "- Process not exist!";
            echo "$rbuf\n";
            writelog("<< $rbuf");
            continue;
        }
        if (isset($aCmd[2]) && strcasecmp($aCmd[2], 'FORCE') == 0)
            $sig = 9;
        else
            $sig = 15;
        if (!posix_kill($pid, $sig)) {
            $rbuf = "- Failed to kill process!";
            echo "$rbuf\n";
            writelog("<< $rbuf");
            continue;
        }
        sleep(5);
        $rbuf = "+ Process killed! If still exist, please try STOP FORCE!";
        echo "$rbuf\n";
        writelog("<< $rbuf");
        continue;
    }
    if (strcasecmp($cmd, 'CHANNEL') == 0) {
        if (!isset($aCmd[2])) {
            $rbuf = "- Syntax: CHANNEL channel";
            echo "$rbuf\n";
            writelog("<< $rbuf");
            continue;
        }
        $channel = base64_decode($aCmd[2]);
        $shcmd = $root_path.'/scripts/channel.sh '.$channel;
        $pipe = popen($shcmd, 'r');
        if (!$pipe) {
            $rbuf = "- popen($shcmd) failed!";
            echo "$rbuf\n";
            writelog("<< $rbuf");
            continue;
        }
        pclose($pipe);
        saveChannelData($channel);
        $rbuf = "+ Channel changed, please wait!";
        echo "$rbuf\n";
        writelog("<< $rbuf");
        continue;
    }

    $rbuf = "- Unknown command: $cmd";
    echo "$rbuf\n";
    writelog("<< $rbuf");
}
fclose($fp);

exit;

function CheckAuthSMTP($username, $password)
{
    global $aUsers;

    if (!in_array($username, $aUsers)) return false;
    $fp = @fsockopen('localhost', 25);
    if (!$fp) return false;
    while (1) {
        if (feof($fp)) {
            fclose($fp);
            return false;
        }
        $buf = trim(fgets($fp));
        if ($buf[3] != '-') break;
    }
    if (strncmp($buf, '220', 3)) {
        fputs($fp, 'QUIT\n');
        fclose($fp);
        return false;
    }
    fputs($fp, "EHLO localhost\n");
    while (1) {
        if (feof($fp)) {
            fclose($fp);
            return false;
        }
        $buf = trim(fgets($fp));
        if ($buf[3] != '-') break;
    }
    if (strncmp($buf, '250', 3)) {
        fputs($fp, 'QUIT\n');
        fclose($fp);
        return false;
    }
    fputs($fp, "AUTH LOGIN\n");
    while (1) {
        if (feof($fp)) {
            fclose($fp);
            return false;
        }
        $buf = trim(fgets($fp));
        if ($buf[3] != '-') break;
    }
    if (strncmp($buf, '334', 3)) {
        fputs($fp, 'QUIT\n');
        fclose($fp);
        return false;
    }
    fputs($fp, base64_encode($username)."\n");
    while (1) {
        if (feof($fp)) {
            fclose($fp);
            return false;
        }
        $buf = trim(fgets($fp));
        if ($buf[3] != '-') break;
    }
    if (strncmp($buf, '334', 3)) {
        fputs($fp, 'QUIT\n');
        fclose($fp);
        return false;
    }
    fputs($fp, base64_encode($password)."\n");
    while (1) {
        if (feof($fp)) {
            fclose($fp);
            return false;
        }
        $buf = trim(fgets($fp));
        if ($buf[3] != '-') break;
    }
    fputs($fp, 'QUIT\n');
    fclose($fp);
    if (strncmp($buf, '235', 3))
        return false;
    return true;
}

function CheckAuthPOP($username, $password)
{
    global $aUsers;

    if (!in_array($username, $aUsers)) return false;
    $fp = @fsockopen('localhost', 110);
    if (!$fp) return false;
    $buf = fgets($fp);
    if ($buf[0] != '+') {
        fclose($fp);
        return false;
    }
    fputs($fp, "USER $username\n");
    $buf = fgets($fp);
    if ($buf[0] != '+') {
        fclose($fp);
        return false;
    }
    fputs($fp, "PASS $password\n");
    $buf = fgets($fp);
    fclose($fp);
    if ($buf[0] != '+')
        return false;
    return true;
}

function deleteSession($sid)
{
    global $root_path;

    $sid_file = $root_path.'/session/'.$sid.'.session';
    unlink($sid_file);
    return;
}

function getSessionData($sid)
{
    global $root_path;

    if ($sid === '' || $sid === false) {
        writelog("No SID!");
        return false;
    }
    $sid_file = $root_path.'/session/'.$sid.'.session';
    if (!is_file($sid_file)) {
        writelog("SID $sid not exist!");
        return false;
    }
    $fp = @fopen($sid_file, 'rt');
    if (!$fp) {
        writelog("Can't open SID file: $sid_file!");
        return false;
    }
    $buf = fread($fp, filesize($sid_file));
    fclose($fp);
    return unserialize($buf);
}

function saveSessionData($aSession)
{
    global $root_path;

    $s = serialize($aSession);
    $sid = md5($s);
    $sid_file = $root_path.'/session/'.$sid.'.session';
    $fp = @fopen($sid_file, 'wt');
    if (!$fp) {
        writelog("Can't open SID file: $sid_file!");
        return false;
    }
    fputs($fp, $s);
    fclose($fp);
    return $sid;
}

function getServerData()
{
    global $root_path, $default_bandwidth, $default_size;

    $file = $root_path.'/session/server.dat';
    $fp = @fopen($file, 'rt');
    if (!$fp) {
        $aData = array('bandwidth' => $default_bandwidth, 'size' => $default_size);
        saveServerData($aData);
        return $aData;
    }
    $buf = fread($fp, filesize($file));
    fclose($fp);
    return unserialize($buf);
}

function saveServerData($aData)
{
    global $root_path;

    $s = serialize($aData);
    $file = $root_path.'/session/server.dat';
    $fp = @fopen($file, 'wt');
    if (!$fp) {
        writelog("Can't open Server file: $file!");
        return false;
    }
    fputs($fp, $s);
    fclose($fp);
    return true;
}

function getChannelData()
{
    global $root_path, $default_channel;

    $file = $root_path.'/session/channel.dat';
    $fp = @fopen($file, 'rt');
    if (!$fp) {
        saveChannelData($default_channel);
        return $default_channel;
    }
    $buf = fread($fp, filesize($file));
    fclose($fp);
    return unserialize($buf);
}

function saveChannelData($ch)
{
    global $root_path;

    $s = serialize($ch);
    $file = $root_path.'/session/channel.dat';
    $fp = @fopen($file, 'wt');
    if (!$fp) {
        writelog("Can't open Channel file: $file!");
        return false;
    }
    fputs($fp, $s);
    fclose($fp);
    return true;
}

function writelog($buf)
{
    global $root_path;

    $file = $root_path.'/log/'.strftime('%Y%m%d').'.log';
    $fp = @fopen($file, 'at');
    if ($fp != 0) {
        $msg = sprintf('%s [%d] %s', strftime('%T'), getmypid(), $buf);
        fputs($fp, $msg."\n");
        fclose($fp);
    }
    return;
}

?>
